//
// binwach_function.c
//
// Charlieplex LED array hello-world
//
// Anders Haldin
// 22/05/15
// Mentored by Andrew Leek
//
// (c) Massachusetts Institute of Technology 2010
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all 
// liability.
//
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/power.h>

#define LED_COUNT 20
#define DDR_BYTE 0
#define PORT_BYTE 1

#define OVERFLOWS_PER_SECOND 31250      /* nominal, should calibrate */

volatile uint16_t ticks = 0;
uint8_t hours = 18;                 /* arbitrary default time */
uint8_t minutes = 7;
uint8_t seconds = 3;


  uint8_t ddrAMatrix[LED_COUNT][3] = {
  // DDR_BYTE   PORT_BYTE
  { 0b00100010, 0b00000010, 0b00000000 }, // 1s
  { 0b00100100, 0b00000100, 0b00000000 }, // 2s
  { 0b00101000, 0b00001000, 0b00000000 }, // 4s
  { 0b00110000, 0b00010000, 0b00000000 }, // 8s

  { 0b00010010, 0b00000010, 0b00000000 }, // 10s
  { 0b00010100, 0b00000100, 0b00000000 }, // 20s
  { 0b00011000, 0b00001000, 0b00000000 }, // 40s

  { 0b00001010, 0b00000010, 0b00000000 }, // 1m
  { 0b00001100, 0b00000100, 0b00000000 }, // 2m
  { 0b00011000, 0b00010000, 0b00000000 }, // 4m
  { 0b00101000, 0b00100000, 0b00000000 }, // 8m

  { 0b00000110, 0b00000010, 0b00000000 }, // 10m
  { 0b00001100, 0b00001000, 0b00000000 }, // 20m
  { 0b00010100, 0b00010000, 0b00000000 }, // 40m

  { 0b00000110, 0b00000100, 0b00000000 }, // 1h
  { 0b00001010, 0b00001000, 0b00000000 }, // 2h
  { 0b00010010, 0b00010000, 0b00000000 }, // 4h
  { 0b00100010, 0b00100000, 0b00000000 }, //  8h

  { 0b00100100, 0b00100000, 0b00000000 }, // 10h
  { 0b00110000, 0b00100000, 0b00000000 }  // 20h
};

ISR(TIM0_OVF_vect) {
     /* This is going off very frequently, so we should make it speedy */
  ticks++;

}


void updateDisplay()
{
  turnOffAll();

	uint8_t x;

	for( x = 0; x < LED_COUNT; x++ )
	{
    if(ddrAMatrix[x][2] == 1)
    {
      turnOn( x );
    }
  }
}


// Called to turn on any one LED
void turnOn( uint8_t led )
{
    DDRA = ddrAMatrix[led][DDR_BYTE];
    PORTA = ddrAMatrix[led][PORT_BYTE];
}


void turnOffAll()
{
  DDRA = 0b00000000;
}


void init()
{
  	// set clock divider to /1
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
}


// Realtime-clock handling functions
void initTimer0_Clock() {
/* Normal mode, just used for the overflow interrupt */
  TCCR0B |= (1 << CS00);  /* 8 MHz clock = ~31250 overflows per second */
  TIMSK0 |= (1 << TOIE0); /* timer overflow interrupt enable */
}

void everySecond() {
  
  seconds++;
  if (seconds > 59) {
    seconds = 0;
    everyMinute();
  }

}

void everyMinute() {
  
  minutes++;
  if (minutes > 59) {
    minutes = 0;
    everyHour();
  }
 
}

void everyHour() {
  hours++;
  if (hours > 23) {                       /* loop around at end of day */
    hours = 0;
  }
}


  //all time shit goes here
void updateTime()
{ 
    if (ticks >= OVERFLOWS_PER_SECOND)
    { 
      ticks = 0;
      everySecond();
    }

}

void translateTime()
{
  // We have to split our numbers up into their 1's and 10's places - this is because each column of led's needs to be treated as if they are a number fom 0-9
  uint8_t onesSeconds = seconds % 10; // This gets the 1's place using modulus or remainder
  uint8_t tensSeconds = seconds / 10; // This gets the 10's place, technically it would return a decimal, but since the type is uint8_t the decimal gets dropped and we are left with just the 10's place number

  // Same applies here
  uint8_t onesMinutes = minutes % 10;
  uint8_t tensMinutes = minutes / 10;

  // Same applies here
  uint8_t onesHours = hours % 10;
  uint8_t tensHours = hours / 10;

  uint8_t i,j,k,l,m,n; // Declare our counter variables

  // Do seconds 1's
  for (i=0; i<4; i++)
  {
    ddrAMatrix[i][2] = onesSeconds & 1;
    onesSeconds >>= 1;
  }

  // Do seconds 10's
  for (j=0; j<3; j++)
  {
    ddrAMatrix[4+j][2] = tensSeconds & 1;
    tensSeconds >>= 1;
  }


  // Do minutes 1's
  for (k=0; k<4; k++)
  {
    ddrAMatrix[7+k][2] = onesMinutes & 1;
    onesMinutes >>= 1;
  }

  // Do minutes 10's
  for (l=0; l<3; l++)
  {
    ddrAMatrix[11+l][2] = tensMinutes & 1;
    tensMinutes >>= 1;
  }


  // Do hours 1's
  for (m=0; m<4; m++) //18
  {
    ddrAMatrix[14+m][2] = onesHours & 1;
    onesHours >>= 1;
  }

  // Do hours 1's
  for (n=0; n<2; n++) //20
  {
    ddrAMatrix[18+n][2] = tensHours & 1;
    tensHours >>= 1;
  }
}


int main(void)
{
   init();

   initTimer0_Clock();

   sei();
   
   //
   // main loop
   //
   while (1)
   {
   		updateTime();
      translateTime();
   		updateDisplay();  
   }
}
